/*
	AC 3-Phase - An AC 3-Phase Calculator
	Copyright © 2007-2015 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

	AC 3-Phase - browser version 1.0
	25 April, 2015
	Copyright © 2007-2015 Harry Whitfield
	mailto:g6auc@arrl.net
*/

/*global paramOne, paramTwo, paramThree, paramFour,
	headingOne, headingTwo, headingThree, headingFour, clearData, Menu
*/

/*properties
    background, keyCode, onchange, onkeypress, selectedIndex, sqrt, style, title,
    toFixed, value, which
*/

// map custom names onto model names
var voltage     = paramOne;
var current     = paramTwo;
var powerFactor = paramThree;
var power       = paramFour;

var voltageHeading     = headingOne;
var currentHeading     = headingTwo;
var powerFactorHeading = headingThree;
var powerDropHeading   = headingFour;

var outputModeMenu = Menu[0];
var iModeMenu      = Menu[1];

var mode     = 0;	// Circuit Length
var iMode    = 0;	// SI
/*
var modeText =  ['Voltage', 'Current', 'Power Factor', 'Power'];
var iModeText = ['Line Mode', 'Phase Mode'];
*/
var modeValue = 0;
var oldModeValue = 0;
var iModeValue = 0;

var multiplier;

function parseNonNegFloat(s) {
	var v = parseFloat(s);
	if (isNaN(v)) {
		return v;
	}
	if (v >= 0) {
		return v;
	}
	return NaN;
}

function parsePfFloat(s) {
	var v = parseFloat(s);
	if (isNaN(v)) {
		return v;
	}
	if ((v >= 0) && (v <= 1)) {
		return v;
	}
	return NaN;
}

function process() {
	var V, I, pf, P, result; 
	
	switch (mode) {	// P=8, pf=4, C=2, V=1
	case 3:
		V  = parseNonNegFloat(voltage.value);
		I  = parseNonNegFloat(current.value);
		pf = parsePfFloat(powerFactor.value);
		result = multiplier * V * I * pf;
		//print('result: ' + result);
		if (isNaN(result)) {
			power.value = 'Invalid Input';
		} else {
			power.value = result.toFixed(3) + "  (" + (result / 745.7).toFixed(5) + "hp)";
		}
		break;
	
	case 2:
		V = parseNonNegFloat(voltage.value);
		I = parseNonNegFloat(current.value);
		P = parseNonNegFloat(power.value);
		result = P / (multiplier * V * I);
		//print('result: ' + result);
		if (isNaN(result)) {
			powerFactor.value = 'Invalid Input';
		} else if (result > 1.0) {
			powerFactor.value = 'Invalid Output ' + result.toFixed(3);
		} else {
			powerFactor.value = result.toFixed(6);
		}
		break;
				
	case 1:
		V  = parseNonNegFloat(voltage.value);
		pf = parsePfFloat(powerFactor.value);
		P  = parseNonNegFloat(power.value);
		result = P / (multiplier * V * pf);
		//print('result: ' + result);
		if (isNaN(result)) {
			current.value = 'Invalid Input';
		} else {
			current.value = result.toFixed(4);
		}
		break;

	case 0:
		I  = parseNonNegFloat(current.value);
		pf = parsePfFloat(powerFactor.value);
		P  = parseNonNegFloat(power.value);
		result = P / (multiplier * I * pf);
		//print('result: ' + result);
		if (isNaN(result)) {
			voltage.value = 'Invalid Input';
		} else {
			voltage.value = result.toFixed(5);
		}
		break;

	default:
		//beep();
		break;
	}
}

function updatePrefs() {
	mode  = modeValue;
	iMode = iModeValue;
		
	if (iMode === 0) {
		multiplier = Math.sqrt(3.0);
		voltageHeading.value = "Line Voltage (V)";
		voltage.title = "Line Voltage (in volts).";
		currentHeading.value = "Line Current (I)";
		current.title = "Line Current (in amperes).";
		//buildVitality("Resources/star.png",  224, 200, dockText[lineMode]);
	} else {
		multiplier = 3.0;
		voltageHeading.value = "Phase Voltage (Vph)";
		voltage.title = "Phase Voltage (in volts).";
		currentHeading.value = "Phase Current (Iph)";
		current.title = "Phase Current (in amperes).";
		//buildVitality("Resources/delta.png", 224, 200, dockText[lineMode]);
	}
}

outputModeMenu.onchange  = function () {
	modeValue = this.selectedIndex;
	switch (oldModeValue) {
	case 0:
		voltage.style.background = "rgba(255, 255, 255, 1.0)";
		break;
	case 1:
		current.style.background = "rgba(255, 255, 255, 1.0)";
		break;
	case 2:
		powerFactor.style.background = "rgba(255, 255, 255, 1.0)";
		break;
	case 3:
		power.style.background = "rgba(255, 255, 255, 1.0)";
		break;
	}

	switch (modeValue) {
	case 0:
		voltage.style.background = "rgba(192, 192, 192, 1.0)";
		break;
	case 1:
		current.style.background = "rgba(192, 192, 192, 1.0)";
		break;
	case 2:
		powerFactor.style.background = "rgba(192, 192, 192, 1.0)";
		break;
	case 3:
		power.style.background = "rgba(192, 192, 192, 1.0)";
		break;
	}
	
	oldModeValue = modeValue;
	updatePrefs();
};

iModeMenu.onchange  = function () {
	iModeValue = this.selectedIndex;
	updatePrefs();
};

voltage.onkeypress = function (event) {
	var x = event.which || event.keyCode;
	
	if (x === 13) { process(); }
};

current.onkeypress = function (event) {
	var x = event.which || event.keyCode;
	
	if (x === 13) { process(); }
};

powerFactor.onkeypress = function (event) {
	var x = event.which || event.keyCode;
	
	if (x === 13) { process(); }
};

power.onkeypress = function (event) {
	var x = event.which || event.keyCode;
	
	if (x === 13) { process(); }
};

updatePrefs();
